'****************************************************************
'*  Name    : Amicus_ADC.inc                                    *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 09/02/2016                                        *
'*  Version : 1.0 Based on Les's Amicus18_ADC.inc code          *
'*  Notes   : ADC library for PIC K20/K22 28 pins series        *
'*          :                                                   *
'****************************************************************
'
'-------------------------------------------------------------------------------
'===============================================================================
' This program is a free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es un software libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
'-------------------------------------------------------------------------------
$ifndef __ADC_INC
$define __ADC_INC
'
' Uncomment this define to speed up the ReadADC() code. (More speed & more code)
'$define _ReadADCHighSpeed_

    GoTo Over_ADC_INC

' Definition of the ADC result variables.
    Dim ADC_ResultW As ADRESL.Word
    Dim ADC_ResultBH As ADRESH
    Dim ADC_ResultBL As ADRESL    

' ADC routines for Amicus18 Board
'===============================================================================
'===============================================================================
'-------------------------------------------------------------------------------
' *** Interrupt enable/disable ***
'-------------------------------------------------------------------------------
' Enable the ADC interrupt enable and the peripherals enable.
' Input : none
' Output: none
' Sintax: ADC_IntEnable()
' Notes :
'
$define ADC_IntEnable()  '
    PIR1bits_ADIF = 0    '
    INTCONbits_PEIE = 1  '
    PIE1bits_ADIE = 1

'-------------------------------------------------------------------------------
' *** Disable the ADIE macro Macro ***
'-------------------------------------------------------------------------------
' Disable the ADC interrupt enable bit.
' Input : none
' Output: PIE1bits_ADIE
' Sintax: ADC_IntDisable()
' Notes :
'
$define ADC_IntDisable() PIE1bits_ADIE = 0

'-------------------------------------------------------------------------------
' *** Start ADC Conversion Macro ***
'-------------------------------------------------------------------------------
' Start the ADC Conversion.
' Input : none
' Output: ADCON0bits_GO_DONE
' Sintax: StartConvADC()
' Notes :
'
$define StartConvADC() ADCON0bits_GO_DONE = 1

'===============================================================================
'===============================================================================
' *** Code for 18F23K20 or 18F24K20 Or 18F25K20 Or 18F26K20 ***
$if (_device = _18F23K20) Or (_device = _18F24K20) Or (_device = _18F25K20) Or (_device = _18F26K20)  
'===============================================================================
' PIC18XXK20
(* ***** Clock source ***** /Num8 *)
$define ADC_FOSC_2    %11111000 ' A/D clock source Fosc/2
$define ADC_FOSC_8    %11111001 ' A/D clock source Fosc/8
$define ADC_FOSC_32   %11111010 ' A/D clock source Fosc/32
$define ADC_FOSC_FRC1 %11111011 ' A/D clock source Internal RC OSC  (clock derived from a dedicated internal oscillator = 600 kHz nominal)
$define ADC_FOSC_4    %11111100 ' A/D clock source Fosc/4
$define ADC_FOSC_16   %11111101 ' A/D clock source Fosc/16
$define ADC_FOSC_64   %11111110 ' A/D clock source Fosc/64
$define ADC_FOSC_FRC2 %11111111 ' A/D clock source Internal RC OSC  (clock derived)

'-------------------------------------------------------------------------------
' PIC18XXK20
(* ***** Acquisition time ***** /Num8 *)
$define ADC_0_TAD     %11000111
$define ADC_2_TAD     %11001111
$define ADC_4_TAD     %11010111
$define ADC_6_TAD     %11011111
$define ADC_8_TAD     %11100111
$define ADC_12_TAD    %11101111
$define ADC_16_TAD    %11110111
$define ADC_20_TAD    %11111111

'-------------------------------------------------------------------------------
' PIC18XXK20
(* ***** Result justification ***** /Num8 *)
$define ADC_RIGHT_JUST %11111111 ' Right justify A/D result
$define ADC_LEFT_JUST  %01111111 ' Left justify A/D result

'-------------------------------------------------------------------------------
' PIC18XXK20
(* ***** Voltage reference ***** /Num8 *)
$define ADC_REF_VDD_VSS             %01001111 ' ADC voltage source VREF+ = VDD and VREF- = VSS
$define ADC_REF_VREFPLUS_VSS        %01011111 ' ADC voltage source VREF+ = ext. source at VREF+ and VREF- = VSS
$define ADC_REF_VDD_VREFMINUS       %01101111 ' ADC voltage source VREF+ = VDD and VREF- = ext. source at VREF-
$define ADC_REF_VREFPLUS_VREFMINUS  %01111111 ' ADC voltage source VREF+ = ext. source at VREF+ and VREF- = ext. source at VREF-

'-------------------------------------------------------------------------------
' PIC18XXK20
(* ***** Analogue Pin Selection AN0 to ANn ***** /Num32 *)
$define ADC_XANA   0x000000  '%00000000 00000000 00000000         ' All digital
$define ADC_0ANA   0x000101  '%00000000 00000001 00000001         ' analogue: AN0
$define ADC_1ANA   0x000303  '%00000000 00000011 00000011         ' analogue: AN0 to AN1
$define ADC_2ANA   0x000707  '%00000000 00000111 00000111         ' analogue: AN0 to AN2
$define ADC_3ANA   0x000F0F  '%00000000 00001111 00001111         ' analogue: AN0 to AN3
$define ADC_4ANA   0x001F2F  '%00000000 00011111 00101111         ' analogue: AN0 to AN4
'$define ADC_5ANA      %0000000000111111         ' analogue: AN0 to AN5 (Not present)
'$define ADC_6ANA      %0000000001111111         ' analogue: AN0 to AN6 (Not present)
'$define ADC_7ANA      %0000000011111111         ' analogue: AN0 to AN7 (Not present)
$define ADC_8ANA   0x0104FF  '%00000001 00000100 11111111        ' analogue: AN0 to AN8
$define ADC_9ANA   0x030CFF  '%00000011 00001100 11111111        ' analogue: AN0 to AN9
$define ADC_10ANA  0x070EFF  '%00000111 00001110 11111111        ' analogue: AN0 to AN10 
$define ADC_11ANA  0x0F1EFF  '%00001111 00011110 11111111        ' analogue: AN0 to AN11
$define ADC_12ANA  0x1F1FFF  '%00011111 00011111 11111111        ' analogue: AN0 to AN12

'-------------------------------------------------------------------------------
' PIC18XXK20
(* ***** Analog individual PORT pin selection ***** /Num32 *)
$define ADC_XANA_ONLY   0x000000  '%00000000 00000000 00000000  ' All digital
$define ADC_0ANA_ONLY   0x800101  '%10000000 00000001 00000001  ' analogue: AN0 only
$define ADC_1ANA_ONLY   0x800202  '%10000000 00000010 00000010  ' analogue: AN1 only
$define ADC_2ANA_ONLY   0x800404  '%10000000 00000100 00000100  ' analogue: AN2 only
$define ADC_3ANA_ONLY   0x800808  '%10000000 00001000 00001000  ' analogue: AN3 only
$define ADC_4ANA_ONLY   0x801020  '%10000000 00010000 00100000  ' analogue: AN4 only
'$define ADC_5ANA_ONLY  %0000000000100000         ' analogue: AN5 only (Not present)
'$define ADC_6ANA_ONLY  %0000000001000000         ' analogue: AN6 only (Not present)
'$define ADC_7ANA_ONLY  %0000000010000000         ' analogue: AN7 only (Not present)
$define ADC_8ANA_ONLY   0x8104FF  '%10000001 00000100 11111111  ' analogue: AN8 only
$define ADC_9ANA_ONLY   0x8208FF  '%10000010 00001000 11111111  ' analogue: AN9 only
$define ADC_10ANA_ONLY  0x8402FF  '%10000100 00000010 11111111  ' analogue: AN10 only 
$define ADC_11ANA_ONLY  0x8810FF  '%10001000 00010000 11111111  ' analogue: AN11 only
$define ADC_12ANA_ONLY  0x9001FF  '%10010000 00000001 11111111  ' analogue: AN12 only

'-------------------------------------------------------------------------------
' PIC18XXK20
(* ***** Analog PORT selection ***** /Num32 *)
$define ADC_PORTA_ANA  0xAA0000 ' analogue: PORTA
$define ADC_PORTB_ANA  0xBB0000 ' analogue: PORTB

'-------------------------------------------------------------------------------
' PIC18XXK20
(* ***** Channel selection ***** /Num8 *)
$define ADC_CH0  %00000000  ' Channel 0
$define ADC_CH1  %00000100  ' Channel 1
$define ADC_CH2  %00001000  ' Channel 2
$define ADC_CH3  %00001100  ' Channel 3
$define ADC_CH4  %00010000  ' Channel 4
'$define ADC_CH5  %00010111  ' Channel 5 (Not present)
'$define ADC_CH6  %00011011  ' Channel 6 (Not present)
'$define ADC_CH7  %00011111  ' Channel 7 (Not present)
$define ADC_CH8  %00100000  ' Channel 8
$define ADC_CH9  %00100100  ' Channel 9
$define ADC_CH10 %00101000  ' Channel 10
$define ADC_CH11 %00101100  ' Channel 11
$define ADC_CH12 %00110000  ' Channel 12

'===============================================================================
' Open the ADC K20
'-------------------------------------------------------------------------------
' Open the ADC peripheral.
' Input : none
' Output: ADCON0, ADCON1, ADCON2, TRIS pins, Analog pins.
' Sintax: OpenADC(Configure ADCON2, Configure ADCON0, Configure ADCON1, Configure Port pins to Analog)
' Notes : Configure first the Analog pin to digital input saves power. See the PIC datasheet.
'
$define OpenADC(pConfig, pConfig2, pConfig3, pPortConfig) _OpenADC pConfig, pConfig2, pConfig3, pPortConfig

_OpenADC Macro- pConfig, pConfig2, pConfig3, pPortConfig
    #if (Prm_Count != 4)
        #error "OpenADC must have 4 Parameters (pConfig , pConfig2, pConfig3, pPortConfig)"
        Exitm
    #endif
    #if ((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "OpenADC Parameter 1 must be constant values"
        Exitm
    #endif
    #if ((Prm_2 != Num8) && (Prm_2 != Num16))
        #error "OpenADC Parameter 2 must be constant values"
        Exitm
    #endif
    #if ((Prm_3 != Num8) && (Prm_3 != Num16))
        #error "OpenADC Parameter 3 must be constant values"
        Exitm
    #endif
    #if ((Prm_4 != Num8) && (Prm_4 != Num16) && (Prm_4 != Num32))
        #error "OpenADC Parameter 4 must be constant values Num32"
        Exitm
    #endif

    Num_SFR (pConfig2 & 0x3C),ADCON0    ' A/D Channels selection.

    Num_SFR (pConfig3 & 0x30),ADCON1    ' Voltage Reference Configuration bits

    ' A/D Conversion Result Format Select bit.
    ' A/D Acquisition time select bits.
    ' A/D Conversion Clock Select bits.
'    Num_SFR (pConfig & 128 | (pConfig & 0x07) | (pConfig & 0x38)),ADCON2
    Num_SFR (pConfig),ADCON2
    '---------------------------------------------------------------------------
    ' Configure pins to Analogue: AN0 to AN4
    #if (pPortConfig <= 0x001F2F)
    Num_Byte (pPortConfig & 0x2F), TRISA
    Num_Byte ((pPortConfig >> 8) & 0x1F), ANSEL
    ANSELH = 0
    #endif

    ' Configure pins To Analogue: AN0 To AN12
    #if ((pPortConfig >= 0x0104FF) && (pPortConfig <= 0x1F1FFF))
    TRISA = $2F
    ANSEL = $1F
    Num_Byte ((pPortConfig >> 8) & 0x1F), TRISB
    Num_Byte ((pPortConfig >> 16) & 0x1F), ANSELH
    #endif
    '---------------------------------------------------------------------------
    ' Configure pins To Analogue: INDIVIDUAL AN0 to AN4
    #if ((pPortConfig >= 0x800101) && (pPortConfig <= 0x801020))
    Num_Byte (pPortConfig & 0x2F), TRISA
    Num_Byte ((pPortConfig >> 8) & 0x1F), ANSEL
    #endif

    ' Configure pins To Analogue: INDIVIDUAL AN8 to AN12
    #if ((pPortConfig >= 0x8104FF) && (pPortConfig <= 0x9001FF))
    Num_Byte ((pPortConfig >> 8) & 0xFF), TRISB
    Num_Byte ((pPortConfig >> 16) & 0xFF), ANSELH
    #endif
    '---------------------------------------------------------------------------
    ' Configure pins To Analog PORT detection 
    ' PORTA
    #if (pPortConfig == 0xAA0000)
        TRISA = $2F
        ANSEL = $1F
        ANSELH = 0
    #endif
    ' PORTB
    #if (pPortConfig == 0xBB0000)
        ANSEL = 0
        TRISB = $3F
        ANSELH = $1F
    #endif
    '---------------------------------------------------------------------------
    ADCON0bits_ADON = 1         ' ADC is Enabled
    '
    ' Dummy read, as per errata workaround
    '
    ADCON0bits_GO_DONE = 1      ' Starts ADC conversion.
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    Btfsc ADCON0bits_GO_DONE    ' Check if the result is ready.
    Bra($ - 2)
Endm

'-------------------------------------------------------------------------------
' *** CloseADC Macro  K20 ***
'-------------------------------------------------------------------------------
' Close the ADC peripheral.
' Input : none
' Output: ADCON0bits_ADON, PIE1bits_ADIE.
' Sintax: CloseADC()
' Notes :
'
$define CloseADC()      '
    ADCON0bits_ADON = 0 '
    PIE1bits_ADIE = 0

'-------------------------------------------------------------------------------
' *** SelChanConvADC Macro K20 ***
'-------------------------------------------------------------------------------
' Selection of the analog channel and Start the ADC Conversion.
' Input : none
' Output: 
' Sintax: SelChanConvADC()
' Notes :
'
$define SelChanConvADC(pChannel) '
    PRODH = pChannel             '
    _SelChanConvADC

_SelChanConvADC Macro-
    GoSub SetChanADC_Sub
    ADCON0bits_GO_DONE = 1
Endm

'-------------------------------------------------------------------------------
' *** SetChanADC Macro K20 ***
'-------------------------------------------------------------------------------
' Selection of the analog channel only.
' Input : none
' Output: 
' Sintax: SetChanADC()
' Notes : Using PRODH as temporaly variable
'
$define SetChanADC(pChannel) '
    PRODH = pChannel         '
    _SetChanADC

_SetChanADC Macro-
    GoSub SetChanADC_Sub
Endm

#ifMacro- _SetChanADC, _SelChanConvADC
SetChanADC_Sub:
    ADCON0 = ADCON0 & $C3
    WREG = PRODH & $3C
    ADCON0 = ADCON0 | WREG
    Return
#endIfMacro-

'-------------------------------------------------------------------------------
' *** BusyADC Macro K20 ***
'-------------------------------------------------------------------------------
' Check if the ADC conversion is runnig.
' Input : none
' Output: Return a Bit. Busy = 1 , NO Busy = 0.
' Sintax: BusyADC()
' Notes : Using a Byte for Return_Var is more efficient.
'
$define BusyADC() _BusyADC

_BusyADC Macro-
    #if(_BusyADC_Return == 1)
        #ifdef WatchDog_Req
            Clrwdt
        #endif
        #if (Return_Type == Byte)
            Movlw 0
            Btfsc ADCON0bits_GO_DONE
            Movlw 1
            Wreg_Byte Return_Var
        #endif
        #if (Return_Type == Word)
            Movlw 0
            Btfsc ADCON0bits_GO_DONE
            Movlw 1
            Wreg_Word Return_Var
        #endif
        #if (Return_Type == Dword)
            Movlw 0
            Btfsc ADCON0bits_GO_DONE
            Movlw 1
            Wreg_Dword Return_Var
        #endif
        #if (Return_Type == Bit)
            If ADCON0bits_GO_DONE = 1 Then
                @Bsf Return_Var,RETURN_BITNUM
            Else
                @Bcf Return_Var,RETURN_BITNUM
            EndIf
        #endif
    #endif
Endm

'-------------------------------------------------------------------------------
' *** ReadADC Macro K20 ***
'-------------------------------------------------------------------------------
' Read an ADC channel.
' Input : channel or not.
' Output: Return a Byte or a Word
' Sintax: ReadADC(Address Channel)
' Notes : If there is not a channel input, the conversion uses the default channel. 
'
$define ReadADC(pCh) _ReadADC pCh

_ReadADC Macro- pCh \ Word
    #if (Prm_Count == 0)
        $ifdef _ReadADCHighSpeed_
            ADCON0bits_GO_DONE = 1
            #ifdef WatchDog_Req
                Clrwdt
            #endif
            Btfsc ADCON0bits_GO_DONE
            Bra($ - 2)
        $else
            GoSub _ReadADC2_Sub        
        $endif

        #if (_ReadADC_Return == 1)
            #if (Return_Type == Byte)
                Byte_Byte ADRESH,Return_Var
            #endif
            #if (Return_Type == Word)
                #if (Return_Var != ADRESL)
                    Byte_Byte ADRESL,Return_Var
                    Byte_Byte ADRESH,Return_Var + 1
                #endif
            #endif
            #if (Return_Type == Dword)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
                Num_Word 0,Return_Var + 2
            #endif
        #endif
        Exitm
    #endif

    #if (Prm_Count > 1)
        #error "ReadADC must have only 1 Parameter"
        Exitm
    #endif
    #if ((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "ReadADC Parameter must be constant value"
        Exitm
    #endif
    #if (Prm_1 == Num8) || (Prm_1 == Num16)
        Num_Byte pCh, PRODH
    #endif

    $ifdef _ReadADCHighSpeed_
        Movlw $C3
        Andwf ADCON0,W
        Iorlw (pChannel & 0x3C)
        Movwf ADCON0
        ADCON0bits_GO_DONE = 1
        #ifdef WatchDog_Req
        Clrwdt
        #endif
        Btfsc ADCON0bits_GO_DONE
        Bra($ - 2)
    $else
        GoSub _ReadADC1_Sub    
    $endif

    #if (_ReadADC_Return == 1)
        #if (Return_Type == Byte)
            Byte_Byte ADRESH,Return_Var
        #endif
        #if (Return_Type == Word)
            #if (Return_Var != ADRESL)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
            #endif
        #endif
        #if (Return_Type == Dword)
            Byte_Byte ADRESL,Return_Var
            Byte_Byte ADRESH,Return_Var + 1
            Num_Word 0,Return_Var + 2
        #endif
    #endif
Endm

$ifndef _ReadADCHighSpeed_
#ifMacro- _ReadADC
_ReadADC1_Sub:
    ADCON0 = ADCON0 & $C3
    WREG = PRODH & $3C
    ADCON0 = ADCON0 | WREG
_ReadADC2_Sub:
    ADCON0bits_GO_DONE = 1
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    Btfsc ADCON0bits_GO_DONE
    Bra($ - 2)
    Return
#endIfMacro-
$endif    
'-------------------------------------------------------------------------------
' *** ADC_Result Macro K20 ***
'-------------------------------------------------------------------------------
' Read the ADC result registers.
' Input : 
' Output: Return a Byte or a Word
' Sintax: ReadADC(Address Channel)
' Notes :  
'
$define MoveADCResult() _ReadADCResultV 

_ReadADCResultV Macro-
'    #ifdef WatchDog_Req
'        Clrwdt
'    #endif
    #if(_ReadADCResultV_Return == 1)
        #if (Return_Type == Byte)
            Byte_Byte ADRESH,Return_Var
        #endif
        #if(Return_Type == Word)
            #if (Return_Var != ADRESL)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
            #endif
        #endif
        #if (Return_Type == Dword)
            Byte_Byte ADRESL,Return_Var
            Byte_Byte ADRESH,Return_Var + 1
            Num_Word 0,Return_Var + 2
        #endif
    #endif
Endm

'-------------------------------------------------------------------------------
$endif  ' PICs K20 

'*******************************************************************************
'*******************************************************************************
'===============================================================================
' ** Code for 18F23K22 ** AND For 18F24K22 18F25K22 ** And For 18F25K22 ** And For 18F26K22 **
$if (_device = _18F23K22) Or (_device = _18F24K22) Or (_device = _18F25K22) Or (_device = _18F26K22) 
'===============================================================================
' PIC18xxK22
(* ***** Clock source ***** /Num8 *)
$define ADC_FOSC_2    %11111000 ' A/D clock source Fosc/2
$define ADC_FOSC_8    %11111001 ' A/D clock source Fosc/8
$define ADC_FOSC_32   %11111010 ' A/D clock source Fosc/32
$define ADC_FOSC_FRC1 %11111011 ' A/D clock source Internal RC OSC  (clock derived from a dedicated internal oscillator = 600 kHz nominal)
$define ADC_FOSC_4    %11111100 ' A/D clock source Fosc/4
$define ADC_FOSC_16   %11111101 ' A/D clock source Fosc/16
$define ADC_FOSC_64   %11111110 ' A/D clock source Fosc/64
$define ADC_FOSC_FRC2 %11111111 ' A/D clock source Internal RC OSC  (clock derived)

'-------------------------------------------------------------------------------
' PIC18xxK22
(* ***** Result justification ***** /Num8 *)
$define ADC_RIGHT_JUST %11111111 ' Right justify A/D result
$define ADC_LEFT_JUST  %01111111 ' Left justify A/D result

'-------------------------------------------------------------------------------
' PIC18xxK22
(* ***** Acquisition time ***** /Num8 *)
$define ADC_0_TAD  %11000111
$define ADC_2_TAD  %11001111
$define ADC_4_TAD  %11010111
$define ADC_6_TAD  %11011111
$define ADC_8_TAD  %11100111
$define ADC_12_TAD %11101111
$define ADC_16_TAD %11110111
$define ADC_20_TAD %11111111

'-------------------------------------------------------------------------------
' PIC18xxK22
(* Positive Voltage Reference Configuration /Num8 *)
$define _ADC_REF_VDD_VDD      0xF3     ' %11110011 = ADC voltage source VREF+ = AVDD
$define _ADC_REF_VDD_VREFPLUS 0xF7     ' %11110111 = ADC voltage source VREF+ connected To external pin, VREF+
$define _ADC_REF_VDD_FVREF    0xFB     ' %11111011 = A/D VREF+ connected to internal signal, FVR BUF2
                                          
'-------------------------------------------------------------------------------
' PIC18xxK22
(* Negative Voltage Reference Configuration /Num8 *)
$define _ADC_REF_VSS_VSS       0xFC     ' %11111100 ADC voltage source VREF- = AVSS
$define _ADC_REF_VSS_VREFMINUS 0xFD     ' %11111101 ADC voltage source VREF- = ext. source at VREF-

' PIC18xxK22
' Combined to make 3 parameters instead of the four required for OpenADC()
$define ADC_REF_VDD_VREFMINUS _ADC_REF_VDD_VDD & _ADC_REF_VSS_VREFMINUS
$define ADC_REF_VDD_VSS _ADC_REF_VDD_VDD & _ADC_REF_VSS_VSS
$define ADC_REF_VREFPLUS_VREFMINUS _ADC_REF_VDD_VREFPLUS & _ADC_REF_VSS_VREFMINUS
$define ADC_REF_VREFPLUS_VSS _ADC_REF_VDD_VREFPLUS & _ADC_REF_VSS_VSS

'-------------------------------------------------------------------------------
' PIC18xxK22
(* ***** Analogue Pins Selection ***** /Num32 *)
$define ADC_XANA      0x000000 '%00000000 00000000 00000000 ' All digital
$define ADC_0ANA      0x000001 '%00000000 00000000 00000001 ' analogue: AN0
$define ADC_1ANA      0x000003 '%00000000 00000000 00000011 ' analogue: AN0 to AN1
$define ADC_2ANA      0x000007 '%00000000 00000000 00000111 ' analogue: AN0 to AN2
$define ADC_3ANA      0x00000F '%00000000 00000000 00001111 ' analogue: AN0 to AN3
$define ADC_4ANA      0x00002F '%00000000 00000000 00101111 ' analogue: AN0 to AN4

$define ADC_8ANA      0x0404FF '%00000100 00000100 11111111 ' analogue: AN0 to AN8
$define ADC_9ANA      0x0C0CFF '%00001100 00001100 11111111 ' analogue: AN0 to AN9
$define ADC_10ANA     0x0E0EFF '%00001110 00001110 11111111 ' analogue: AN0 to AN10
$define ADC_11ANA     0x1E1EFF '%00011110 00011110 11111111 ' analogue: AN0 to AN11
$define ADC_12ANA     0x1F1FFF '%00011111 00011111 11111111 ' analogue: AN0 to AN12
$define ADC_13ANA     0x3F3FFF '%00111111 00111111 11111111 ' analogue: AN0 to AN13

$define ADC_14ANA     0x010000 '%00000001 11111111 11111111 ' analogue: AN0 to AN14
$define ADC_15ANA     0x030000 '%00000011 11111111 11111111 ' analogue: AN0 to AN15
$define ADC_16ANA     0x070000 '%00000111 11111111 11111111 ' analogue: AN0 to AN16
$define ADC_17ANA     0x0F0000 '%00001111 11111111 11111111 ' analogue: AN0 to AN17
$define ADC_18ANA     0x1F0000 '%00011111 11111111 11111111 ' analogue: AN0 to AN18
$define ADC_19ANA     0x3F0000 '%00111111 11111111 11111111 ' analogue: AN0 to AN19

' PIC18xxK22
(* ***** Analog individual PORT pin selection ***** /Num32 *)
$define ADC_XANA_ONLY   0x800000 '%10000000 00000000 00000000 ' All digital
$define ADC_0ANA_ONLY   0x800001 '%10000000 00000000 00000001 ' analogue: AN0 only
$define ADC_1ANA_ONLY   0x800002 '%10000000 00000000 00000010 ' analogue: AN1 only
$define ADC_2ANA_ONLY   0x800004 '%10000000 00000000 00000100 ' analogue: AN2 only
$define ADC_3ANA_ONLY   0x800008 '%10000000 00000000 00001000 ' analogue: AN3 only
$define ADC_4ANA_ONLY   0x800020 '%10000000 00000000 00100000 ' analogue: AN4 only    

$define ADC_8ANA_ONLY   0x8404FF '%10000100 00000100 11111111 ' analogue: AN8 only
$define ADC_9ANA_ONLY   0x8808FF '%10001000 00001000 11111111 ' analogue: AN9 only
$define ADC_10ANA_ONLY  0x8202FF '%10000010 00000010 11111111 ' analogue: AN10 only
$define ADC_11ANA_ONLY  0x9010FF '%10010000 00010000 11111111 ' analogue: AN11 only
$define ADC_12ANA_ONLY  0x8101FF '%10000001 00000001 11111111 ' analogue: AN12 only
$define ADC_13ANA_ONLY  0xA020FF '%10100000 00100000 11111111 ' analogue: AN13 only

$define ADC_14ANA_ONLY  0xC1FFFF '%11000001 11111111 11111111 ' analogue: AN14 only
$define ADC_15ANA_ONLY  0xC2FFFF '%11000010 11111111 11111111 ' analogue: AN15 only
$define ADC_16ANA_ONLY  0xC4FFFF '%11000100 11111111 11111111 ' analogue: AN16 only
$define ADC_17ANA_ONLY  0xC8FFFF '%11001000 11111111 11111111 ' analogue: AN17 only
$define ADC_18ANA_ONLY  0xD0FFFF '%11010000 11111111 11111111 ' analogue: AN18 only
$define ADC_19ANA_ONLY  0xE0FFFF '%11100000 11111111 11111111 ' analogue: AN19 only

' PIC18xxK22
(* ***** Analog PORT selection ***** /Num32 *)
$define ADC_PORTA_ANA   0xAAA000 ' analogue: PORTA
$define ADC_PORTB_ANA   0xBBB000 ' analogue: PORTB
$define ADC_PORTC_ANA   0xCCC000 ' analogue: PORTC
'-------------------------------------------------------------------------------
' PIC18xxK22
(* ***** Channel selection ***** /Byte *) 
$define ADC_CH0  %00000000  ' Channel 0
$define ADC_CH1  %00000100  ' Channel 1
$define ADC_CH2  %00001000  ' Channel 2
$define ADC_CH3  %00001100  ' Channel 3
$define ADC_CH4  %00010000  ' Channel 4
'$define ADC_CH5  %00010100  ' Channel 5 (Not present)
'$define ADC_CH6  %00011000  ' Channel 6 (Not present)
'$define ADC_CH7  %00011100  ' Channel 7 (Not present)
$define ADC_CH8  %00100000  ' Channel 8
$define ADC_CH9  %00100100  ' Channel 9
$define ADC_CH10 %00101000  ' Channel 10
$define ADC_CH11 %00101100  ' Channel 11
$define ADC_CH12 %00110000  ' Channel 12
$define ADC_CH13 %00110100  ' Channel 13
$define ADC_CH14 %00111000  ' Channel 14
$define ADC_CH15 %00111100  ' Channel 15

$define ADC_CH16 %01000000  ' Channel 16
$define ADC_CH17 %01000100  ' Channel 17
$define ADC_CH18 %01001000  ' Channel 18
$define ADC_CH19 %01001100  ' Channel 19

'===============================================================================
' Open the ADC K22
'-------------------------------------------------------------------------------
' Open the ADC peripheral.  K22
' Input : none
' Output: ADCON0, ADCON1, ADCON2, TRIS pins, Analog pins.
' Sintax: OpenADC(Configure ADCON2, Configure ADCON0, Configure ADCON1, Configure Port pins to Analog)
' Notes : Configure first the Analog pin to digital input saves power. See the PIC datasheet.

$define OpenADC(pConfig, pConfig2, pConfig3, pPortConfig) _OpenADC pConfig, pConfig2, pConfig3, pPortConfig

_OpenADC Macro- pConfig, pConfig2, pConfig3, pPortConfig
    #if (Prm_Count != 4)
        #error "OpenADC for 18F25K22 must have 4 Parameters (pConfig, pConfig2, pConfig3, pPortConfig)"
        Exitm
    #endif
    #if ((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "OpenADC Parameter 1 must be constant values"
        Exitm
    #endif
    #if ((Prm_2 != Num8) && (Prm_2 != Num16))
        #error "OpenADC Parameter 2 must be constant values"
        Exitm
    #endif
    #if ((Prm_3 != Num8) && (Prm_3 != Num16))
        #error "OpenADC Parameter 3 must be constant values"
        Exitm
    #endif
    #if ((Prm_4 != Num8) && (Prm_4 != Num16) && (Prm_4 != Num32))
        #error "OpenADC Parameter 4 must be constant values"
        Exitm
    #endif

    ' Configure the Analog Channel Select bits
    Num_SFR (pConfig2 & 0x7C),ADCON0

    ' Configure the Positive and Negative Voltage Reference Configuration bits
    Num_SFR (pConfig3 & 0x0F),ADCON1

    ' A/D Conversion Result Format Select bit = 1bit
    ' A/D Acquisition time select bits = 3 bits.
    ' A/D Conversion Clock Select bits = 3 bits.
    Num_SFR ((pConfig & 0x07) | (pConfig & 128) | (pConfig & 0x38)),ADCON2
    '---------------------------------------------------------------------------
    ' Configure the Analog pins AN0 To AN4 (last PORTA)
    #if (pPortConfig <= 0x00002F)
        Num_Byte (pPortConfig & 0x3F), TRISA
        Num_Byte (pPortConfig & 0x3F), ANSELA
        ANSELB = 0
        ANSELC = 0
    #endif

    ' Configure the Analog pins AN0 To AN13 (last PORTB)
    #if ((pPortConfig >= 0x0404FF) && (pPortConfig <= 0x3F3FFF))
        TRISA = $2F
        Num_Byte (pPortConfig >> 8), TRISB
        ANSELA = $2F
        Num_Byte (pPortConfig >> 16), ANSELB
        ANSELC = 0
    #endif

    ' Configure the Analog pins AN0 To AN19  (last PORTC)
    #if ((pPortConfig >= 0x010000) && (pPortConfig <= 0x3F0000))
        TRISA = $2F
        TRISB = $3F
        Num_Byte (pPortConfig >> 14), TRISC
        ANSELA = $2F
        ANSELB = $3F
        Num_Byte (pPortConfig >> 14), ANSELC
    #endif
    '---------------------------------------------------------------------------
    ' Configure the Analog PORTA individual bits
    #if ((pPortConfig >= 0x800001) && (pPortConfig <= 0x800020))
        Num_Byte (pPortConfig & 0x00003F), TRISA
        Num_Byte (pPortConfig & 0x00003F), ANSELA
    #endif

    ' Configure the Analog PORTB individual bits
    #if ((pPortConfig >= 0x8101FF) && (pPortConfig <= 0xA020FF))
        Num_Byte ((pPortConfig >> 8) & 0x00003F), TRISB
        Num_Byte ((pPortConfig >> 16) & 0x00003F), ANSELB
    #endif

    ' Configure the Analog PORTC individual bits
    #if ((pPortConfig >= 0xC1FFFF) && (pPortConfig <= 0xE0FFFF))
        Num_Byte ((pPortConfig >> 14) & 0x0000FC), TRISC
        Num_Byte ((pPortConfig >> 14) & 0x0000FC), ANSELC
    #endif
    '---------------------------------------------------------------------------
    ' For full PORT Analog
    ' PORTA all
    #if (pPortConfig == 0xAAA000)
        TRISA = $2F
        ANSELA = $2F
        ANSELB = 0
        ANSELC = 0
    #endif
    ' PORTB all
    #if (pPortConfig == 0xBBB000)
        ANSELA = 0
        TRISB = $3F
        ANSELB = $3F
        ANSELC = 0
    #endif
    ' PORTC all
    #if (pPortConfig == 0xCCC000)
        TRISC = $FC 
        ANSELA = 0
        ANSELB = 0
        ANSELC = $FC
    #endif
    '---------------------------------------------------------------------------
    ADCON0bits_ADON = 1         ' ADC is enabled.
'
' Dummy read, as per errata workaround
'
    ADCON0bits_GO_DONE = 1      ' Starts ADC conversion.
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    Btfsc ADCON0bits_GO_DONE    ' Check if ADC conversion is done.
    Bra($ - 2)
Endm

'-------------------------------------------------------------------------------
' *** CloseADC Macro K22 ***
'-------------------------------------------------------------------------------
' Close the ADC peripheral.
' Input : none
' Output: ADCON0bits_ADON, PIE1bits_ADIE.
' Sintax: CloseADC()
' Notes :
'
$define CloseADC()      '
    ADCON0bits_ADON = 0 '
    PIE1bits_ADIE = 0

'-------------------------------------------------------------------------------
' *** SelChanConvADC Macro K22 ***
'-------------------------------------------------------------------------------
' Selection of the analog channel and Start the ADC Conversion.
' Input : none
' Output: 
' Sintax: SelChanConvADC()
' Notes :
'
$define SelChanConvADC(pChannel) '
    PRODH = pChannel             '
    _SelChanConvADC

_SelChanConvADC Macro-
    GoSub SetChanADC_Sub
    ADCON0bits_GO_DONE = 1
Endm

'-------------------------------------------------------------------------------
' *** SetChanADC Macro K22 ***
'-------------------------------------------------------------------------------
' Selection of the analog channel only.
' Input : none
' Output: 
' Sintax: SetChanADC()
' Notes :
'
$define SetChanADC(pChannel) '
    PRODH = pChannel         '
    _SetChanADC

_SetChanADC Macro-
    GoSub SetChanADC_Sub
Endm

#ifMacro- _SetChanADC, _SelChanConvADC
SetChanADC_Sub:
'    #ifdef WatchDog_Req
'        Clrwdt
'    #endif
    ADCON0 = ADCON0 & $83
    WREG = PRODH & $7C
    ADCON0 = ADCON0 | WREG
    Return
#endIfMacro-

'-------------------------------------------------------------------------------
' *** BusyADC Macro K22 ***
'-------------------------------------------------------------------------------
' Check if the ADC conversion is runnig.
' Input : none
' Output: Return a Bit. Busy = 1 , NO Busy = 0.
' Sintax: BusyADC()
' Notes : Using a Byte for Return_Var is more efficient.
'
$define BusyADC() _BusyADC

_BusyADC Macro-
    #if(_BusyADC_Return == 1)
'        #ifdef WatchDog_Req
'            Clrwdt
'        #endif
        #if (Return_Type == Byte)
            Movlw 0
            Btfsc ADCON0bits_GO_DONE
            Movlw 1
            Wreg_Byte Return_Var
        #endif
        #if (Return_Type == Word)
            Movlw 0
            Btfsc ADCON0bits_GO_DONE
            Movlw 1
            Wreg_Word Return_Var
        #endif
        #if (Return_Type == Dword)
            Movlw 0
            Btfsc ADCON0bits_GO_DONE
            Movlw 1
            Wreg_Dword Return_Var
        #endif
        #if (Return_Type == Bit)
            If ADCON0bits_GO_DONE = 1 Then
                @Bsf Return_Var, RETURN_BITNUM
            Else
                @Bcf Return_Var, RETURN_BITNUM
            EndIf
        #endif
    #endif
Endm

'-------------------------------------------------------------------------------
' *** ReadADC Macro K22 ***
'-------------------------------------------------------------------------------
' Read an ADC channel.
' Input : channel or none.
' Output: Return a Byte or a Word
' Sintax: ReadADC(Address Channel)
' Notes : If there is not a channel input, the conversion uses the default channel. 
'
$define ReadADC(pChannel) _ReadADC pChannel

_ReadADC Macro- pChannel \ Word
    #if (Prm_Count == 0)
        $ifdef _ReadADCHighSpeed_
            ADCON0bits_GO_DONE = 1
            #ifdef WatchDog_Req
                Clrwdt
            #endif
            Btfsc ADCON0bits_GO_DONE
            Bra($ - 2)
        $else
            GoSub _ReadADC2_Sub        
        $endif
        #if (_ReadADC_Return == 1)
            #if (Return_Type == Byte)
                Byte_Byte ADRESH,Return_Var
            #endif
            #if (Return_Type == Word)
                #if (Return_Var != ADRESL)
                    Byte_Byte ADRESL,Return_Var
                    Byte_Byte ADRESH,Return_Var + 1
                #endif
            #endif
            #if (Return_Type == Dword)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
                Num_Word 0,Return_Var + 2
            #endif
        #endif
        Exitm
    #endif

    #if (Prm_Count > 1)
        #error "ReadADC must have only 1 Parameter"
        Exitm
    #endif
    #if ((Prm_1 != Num8) && (Prm_1 != Num16))
        #error "ReadADC Parameter must be constant value"
        Exitm
    #endif
    #if (Prm_1 == Num8) || (Prm_1 == Num16)
        Num_Byte pChannel, PRODH
    #endif

    $ifdef _ReadADCHighSpeed_
        Movlw $83
        Andwf ADCON0,W
        Iorlw (pChannel & 0x7C)
        Movwf ADCON0
        ADCON0bits_GO_DONE = 1
        #ifdef WatchDog_Req
            Clrwdt
        #endif
        Btfsc ADCON0bits_GO_DONE
        Bra($ - 2)
    $else
        GoSub _ReadADC1_Sub    
    $endif
    #if (_ReadADC_Return == 1)
        #if (Return_Type == Byte)
            Byte_Byte ADRESH,Return_Var
        #endif
        #if (Return_Type == Word)
            #if (Return_Var != ADRESL)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
            #endif
        #endif
        #if (Return_Type == Dword)
            Byte_Byte ADRESL,Return_Var
            Byte_Byte ADRESH,Return_Var + 1
            Num_Word 0,Return_Var + 2
        #endif
    #endif
Endm

$ifndef _ReadADCHighSpeed_
#ifMacro- _ReadADC
_ReadADC1_Sub:
    ADCON0 = ADCON0 & $83
    WREG = PRODH & $7C
    ADCON0 = ADCON0 | WREG
_ReadADC2_Sub:
    ADCON0bits_GO_DONE = 1
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    Btfsc ADCON0bits_GO_DONE
    Bra($ - 2)
    Return
#endIfMacro-
$endif
'-------------------------------------------------------------------------------
' *** ADC_Result Macro K22 ***
'-------------------------------------------------------------------------------
' Read the ADC result registers.
' Input : 
' Output: Return a Byte or a Word
' Sintax: ReadADC(Address Channel)
' Notes :  
'
$define ReadADCResult() _ReadADCResult 

_ReadADCResult Macro-
    #ifdef WatchDog_Req
        Clrwdt
    #endif
    #if (_ReadADCResult_Return == 1)
        #if (Return_Type == Byte)
            Byte_Byte ADRESH,Return_Var
        #endif
        #if(Return_Type == Word)
            #if (Return_Var != ADRESL)
                Byte_Byte ADRESL,Return_Var
                Byte_Byte ADRESH,Return_Var + 1
            #endif
        #endif
        #if (Return_Type == Dword)
            Byte_Byte ADRESL,Return_Var
            Byte_Byte ADRESH,Return_Var + 1
            Num_Word 0,Return_Var + 2
        #endif
    #endif
Endm
'===============================================================================
$endif  ' PICs K22

Over_ADC_INC:
' END Macros
'-------------------------------------------------------------------------------
$endif

